/*****************************************************************************/
/*                      WiNRADiO G69DDC EXCELSIOR ULTRA                      */
/*                                SDK DEMO                                   */
/*                             Main GUI window                               */
/*                        (c) Copyright RADIXON 2022                         */
/*****************************************************************************/

#ifndef __U_MAINWINDOW_H__
#define __U_MAINWINDOW_H__

#include <wx/wx.h>
#include <wx/splitter.h>
#include <wx/sizer.h>
#include <wx/tglbtn.h>
#include <wx/combobox.h>
#include "u_receiver.h"
#include "u_spectrum.h"
#include "u_rxpanel.h"
#include "u_swpspectrum.h"

class TMainWindow:public wxFrame,public TReceiverCallback
{
public:
    TMainWindow(TReceiver *Receiver);
    ~TMainWindow(void);
private:
    TReceiver           *m_Receiver;

    uint32_t            m_ChannelCount;

    wxBoxSizer          *m_MainSizer;
    wxBoxSizer          *m_IFSizerTop;
    wxBoxSizer          *m_IFSizerBottom;
    wxBoxSizer          *m_RXSizer;

    wxToggleButton      **m_BRX;

    wxToggleButton      *m_BSpectrum;
    wxToggleButton      *m_BSweeper;

    TSpectrum           *m_IFSpectrum;
    wxStaticText        *m_LAttenuator;
    wxComboBox          *m_CBAttenuator;
    wxToggleButton      *m_BPreamp;
    wxToggleButton      *m_BMWFilter;

    wxToggleButton      *m_BPower;
    wxStaticText        *m_LRBW;
    wxComboBox          *m_CBIFRBW;

    TSwpSpectrum        *m_SwpSpectrum;
    wxComboBox          *m_CBSwpRBW;
    wxStaticText        *m_LSwpStart;
    TFrequencyEdit      *m_ESwpStart;
    wxStaticText        *m_LSwpEnd;
    TFrequencyEdit      *m_ESwpEnd;
    wxToggleButton      *m_BSwpStart;
    wxButton            *m_BSwpStop;
    wxToggleButton      *m_BSwpLoop;
    wxButton            *m_BSwpClear;

    wxStaticText        *m_LDataRate;
    wxStaticText        *m_LDataRateValue;
    wxStaticText        *m_LLostDataRate;
    wxStaticText        *m_LLostDataRateValue;

    wxStaticText        *m_LFrontEndFrequency;
    wxStaticText        *m_LFrontEndFrequencyValue;
    wxStaticText        *m_LRange;
    wxStaticText        *m_LRangeValue;
    wxButton            *m_BIFDown;
    wxButton            *m_BIFUp;


    wxBoxSizer          *m_DDC1Sizer;
    wxBoxSizer          *m_DDC1SizerTop;
    TSpectrum           *m_DDC1Spectrum;
    wxStaticText        *m_LDDC1BW;
    wxComboBox          *m_CBDDC1BW;
    wxStaticText        *m_LDDC1RBW;
    wxComboBox          *m_CBDDC1RBW;

    TRXPanel            **m_RXPanels;
    G69DDC_DEVICE_INFO  m_DeviceInfo;
    TRXPanel            *m_ActiveRXPanel;
    wxTimer             m_StateTimer;
    wxTimer             m_InitPowerOnTimer;
    uint64_t            m_BytesTransferred;
    uint64_t            m_BytesLost;
    wxLongLong          m_Time;
    wxStopWatch         m_StopWatch;

    bool                m_SwpLoop;

    virtual void ReceiverCallback_Event(TReceiverEvent Event);
    virtual void ReceiverCallback_ChannelEvent(uint32_t Channel,TChannelEvent Event);
    virtual void ReceiverCallback_DDC1Spectrum(float *Data,uint32_t Count,double Range,double Offset);
    virtual void ReceiverCallback_DDC2Spectrum(uint32_t Channel,float *Data,uint32_t Count,double Range,double Offset);
    virtual void ReceiverCallback_AudioSpectrum(uint32_t Channel,float *Data,uint32_t Count,uint32_t Range);
    virtual void ReceiverCallback_IFSpectrum(float *Data,uint32_t Count,double Range,double Offset);
    virtual void ReceiverCallback_Audio(uint32_t Channel,const float *Samples,uint32_t Count);
    virtual void ReceiverCallbask_Sweeper(float *Data,uint32_t Count);
    virtual bool ReceiverCallback_SweeperQueryEnd(void);


    void Initialize(void);
    void UpdateState(void);
    void UpdateSwpState(void);
    void UpdateActiveChannel(void);
    void UpdateAttenuatorState(void);
    void UpdatePreamplifier(void);
    void UpdateMWFilter(void);
    void UpdateIFRBWIndex(void);
    void UpdateRange(void);
    void UpdateFrontEndFrequency(void);

    void UpdateDDC1AbsFrequency(void);
    void UpdateDDC1(void);
    void UpdateDDC1RBWList(void);
    void UpdateDDC1RBW(void);

    void UpdateDDC2(uint32_t ChannelIndex);
    void UpdateDDC2Frequency(uint32_t Channel);

    void UpdateBottomPanel(void);



    void OnButtonClick(wxCommandEvent &Event);
    void OnRXButtonClick(wxCommandEvent &Event);
    void OnComboBoxChanged(wxCommandEvent &Event);
    void OnSpectrumTune(TSpectrumTuneEvent &Event);
    void OnTimer(wxTimerEvent& Event);
    void OnStopSweeper(wxCommandEvent &Event);
    void OnNumericEditChanged(wxNotifyEvent &Event);

    static wxString DataRateToString(double Rate);

    DECLARE_EVENT_TABLE()
};

#endif

